@@ -25,6 +25,7 @@
 #include "statuswidget.h"
 #include "drkonqi.h"
 #include "drkonqi_globals.h"
+#include "applicationdetailsexamples.h"
 
 #include <QtCore/QTimer>
 
@@ -37,6 +38,7 @@
 #include <KMessageBox>
 #include <KToolInvocation>
 #include <KWallet/Wallet>
+#include <kcapacitybar.h>
 
 static const char kWalletEntryName[] = "drkonqi_bugzilla";
 static const char kWalletEntryUsername[] = "username";
@@ -75,7 +77,8 @@ BugzillaLoginPage::BugzillaLoginPage(ReportAssistantDialog * parent) :
                             "<link url='%1'>KDE bug tracking system</link> in order to "
                             "file a bug report, because we may need to contact you later "
                             "for requesting further information. If you do not have "
-                            "one, you can freely <link url='%2'>create one here</link>.</note>",
+                            "one, you can freely <link url='%2'>create one here</link>. "
+                            "Please do not use disposable email accounts.</note>",
                             DrKonqi::crashedApplication()->bugReportAddress(),
                             QLatin1String(KDE_BUGZILLA_CREATE_ACCOUNT_URL)));
 }
@@ -197,6 +200,9 @@ void BugzillaLoginPage::loginClicked()
     if (!(ui.m_userEdit->text().isEmpty() || ui.m_passwordEdit->text().isEmpty())) {
         ui.m_loginButton->setEnabled(false);
 
+        ui.m_userLabel->setEnabled(false);
+        ui.m_passwordLabel->setEnabled(false);
+
         ui.m_userEdit->setEnabled(false);
         ui.m_passwordEdit->setEnabled(false);
         ui.m_savePasswordCheckBox->setEnabled(false);
@@ -283,19 +289,24 @@ BugzillaLoginPage::~BugzillaLoginPage()
 
 BugzillaInformationPage::BugzillaInformationPage(ReportAssistantDialog * parent)
         : ReportAssistantPage(parent),
-        m_textsOK(false), m_distributionComboSetup(false), m_distroComboVisible(false)
+        m_textsOK(false), m_distributionComboSetup(false), m_distroComboVisible(false),
+        m_requiredCharacters(1)
 {
     ui.setupUi(this);
-    
+    m_textCompleteBar = new KCapacityBar(KCapacityBar::DrawTextInline, this);
+    ui.horizontalLayout_2->addWidget(m_textCompleteBar);
+
     connect(ui.m_titleEdit, SIGNAL(textChanged(QString)), this, SLOT(checkTexts()));
     connect(ui.m_detailsEdit, SIGNAL(textChanged()), this, SLOT(checkTexts()));
 
     connect(ui.m_titleLabel, SIGNAL(linkActivated(QString)), this, SLOT(showTitleExamples()));
+    connect(ui.m_detailsLabel, SIGNAL(linkActivated(QString)), this,
+            SLOT(showDescriptionHelpExamples()));
 
     ui.m_compiledSourcesCheckBox->setChecked(
                                     DrKonqi::systemInformation()->compiledSources());
-    
-    checkTexts();
+
+    //ui.m_distributionGroupBox->setVisible(false);  //TODO do this?
 }
 
 void BugzillaInformationPage::aboutToShow()
@@ -356,9 +367,46 @@ void BugzillaInformationPage::aboutToShow()
         m_distributionComboSetup = true;
     }
 
+    //Calculate the minimum number of characters required for a description
+    //If creating a new report: minimum 40, maximum 80
+    //If attaching to an existent report: minimum 30, maximum 50
+    int multiplier = (reportInterface()->attachToBugNumber() == 0) ? 10 : 5;
+    m_requiredCharacters = 20 + (reportInterface()->selectedOptionsRating() * multiplier);
+    
+    //Fill the description textedit with some headings:
+    QString descriptionTemplate;
+    if (ui.m_detailsEdit->toPlainText().isEmpty()) {
+        if (reportInterface()->userCanProvideActionsAppDesktop()) {
+            descriptionTemplate += "- What I was doing when the application crashed:\n\n";
+        }
+        if (reportInterface()->userCanProvideUnusualBehavior()) {
+            descriptionTemplate += "- Unusual behavior I noticed:\n\n";
+        }
+        if (reportInterface()->userCanProvideApplicationConfigDetails()) {
+            descriptionTemplate += "- Custom settings of the application:\n\n";
+        }
+        ui.m_detailsEdit->setText(descriptionTemplate);
+    }
+
     checkTexts(); //May be the options (canDetail) changed and we need to recheck
 }
 
+int BugzillaInformationPage::currentDescriptionCharactersCount()
+{
+    QString description = ui.m_detailsEdit->toPlainText();
+
+    //Do not count template messages, and other misc chars
+    description.remove("What I was doing when the application crashed");
+    description.remove("Unusual behavior I noticed");
+    description.remove("Custom settings of the application");
+    description.remove('\n');
+    description.remove('-');
+    description.remove(':');
+    description.remove(' ');
+
+    return description.size();
+}
+
 void BugzillaInformationPage::checkTexts()
 {
     //If attaching this report to an existing one then the title is not needed
@@ -369,6 +417,19 @@ void BugzillaInformationPage::checkTexts()
     bool ok = !((ui.m_titleEdit->isVisible() && ui.m_titleEdit->text().isEmpty())
         || ui.m_detailsEdit->toPlainText().isEmpty());
 
+    QString message;
+    int percent = currentDescriptionCharactersCount() * 100 / m_requiredCharacters;
+    if (percent >= 100) {
+        percent = 100;
+        message = i18nc("the minimum required length of a text was reached",
+                        "Minimum length reached");
+    } else {
+        message = i18nc("the minimum required length of a text wasn't reached yet",
+                        "Provide more information");
+    }
+    m_textCompleteBar->setValue(percent);
+    m_textCompleteBar->setText(message);
+
     if (ok != m_textsOK) {
         m_textsOK = ok;
         emitCompleteChanged();
@@ -380,12 +441,7 @@ bool BugzillaInformationPage::showNextPage()
     checkTexts();
 
     if (m_textsOK) {
-        //Calculate the minimum number of characters required for a description
-        //If creating a new report: minimum 40, maximum 80
-        //If attaching to an existant report: minimum 30, maximum 50
-        int multiplier = (reportInterface()->attachToBugNumber() == 0) ? 10 : 5;
-        int requiredDescription = 20 + (reportInterface()->selectedOptionsRating() * multiplier);
-        bool detailsShort = ui.m_detailsEdit->toPlainText().size() < requiredDescription;
+        bool detailsShort = currentDescriptionCharactersCount() < m_requiredCharacters;
 
         if (detailsShort) {
             //The user input is less than we want.... encourage to write more
@@ -451,6 +507,37 @@ void BugzillaInformationPage::showTitleExamples()
     QToolTip::showText(QCursor::pos(), titleExamples);
 }
 
+void BugzillaInformationPage::showDescriptionHelpExamples()
+{
+    QString descriptionHelp = i18nc("@info:tooltip help and examples of good bug descriptions",
+                                  "Describe in as much detail as possible the crash circumstances:");
+    if (reportInterface()->userCanProvideActionsAppDesktop()) {
+        descriptionHelp += "<br />" +
+                           i18nc("@info:tooltip help and examples of good bug descriptions",
+                                 "- Detail which actions were you taking inside and outside the "
+                                 "application an instant before the crash.");
+    }
+    if (reportInterface()->userCanProvideUnusualBehavior()) {
+        descriptionHelp += "<br />" + 
+                           i18nc("@info:tooltip help and examples of good bug descriptions",
+                                 "- Note if you noticed any unusual behavior in the application "
+                                 "or in the whole environment.");
+    }
+    if (reportInterface()->userCanProvideApplicationConfigDetails()) {
+        descriptionHelp += "<br />" +
+                           i18nc("@info:tooltip help and examples of good bug descriptions",
+                                 "- Note any non-default configuration in the application.");
+        if (reportInterface()->appDetailsExamples()->hasExamples()) {
+            descriptionHelp += ' ' +
+                               i18nc("@info:tooltip examples of configuration details. "
+                                     "the examples are already translated",
+                                     "Examples: %1",
+                                     reportInterface()->appDetailsExamples()->examples());
+        }
+    }
+    QToolTip::showText(QCursor::pos(), descriptionHelp);
+}
+
 //END BugzillaInformationPage
 
 //BEGIN BugzillaPreviewPage
