@@ -3,7 +3,7 @@
  * $Id: sourceheader 511311 2006-02-19 14:51:05Z trueg $
  *
  * This file is part of the Nepomuk KDE project.
- * Copyright (C) 2006-2007 Sebastian Trueg <trueg@kde.org>
+ * Copyright (C) 2006-2009 Sebastian Trueg <trueg@kde.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,22 +19,19 @@
 #include <QtCore/QMap>
 
 #include <Soprano/Util/SignalCacheModel>
+#include <Soprano/BackendSettings>
 
 
 namespace Soprano {
     class Model;
     class Backend;
-    namespace Index {
-        class IndexFilterModel;
-        class CLuceneIndex;
-    }
 }
 
 class KJob;
 
 namespace Nepomuk {
 
-    class CLuceneAnalyzer;
+    class ModelCopyJob;
 
     class Repository : public Soprano::Util::SignalCacheModel
     {
@@ -52,9 +49,9 @@ namespace Nepomuk {
             OPEN
         };
 
-        State state() const;
+        State state() const { return m_state; }
 
-        static const Soprano::Backend* activeSopranoBackend();
+        QString usedSopranoBackend() const;
 
     public Q_SLOTS:
         /**
@@ -64,41 +61,31 @@ namespace Nepomuk {
 
         void close();
 
-        /**
-         * Calls slotDoOptimize via timer for instant return.
-         */
-        void optimize();
-
     Q_SIGNALS:
         void opened( Repository*, bool success );
 
     private Q_SLOTS:
         void copyFinished( KJob* job );
-        void rebuildingIndexFinished();
-        void slotDoOptimize();
 
     private:
-        /**
-         * \return true if the index needs to be rebuilt and the method
-         * took over initialization.
-         */
-        bool rebuildIndexIfNecessary();
+        Soprano::BackendSettings readVirtuosoSettings() const;
 
         QString m_name;
         State m_state;
 
-        // the base path for the data. Will contain subfolders:
-        // "index" for the fulltext index
-        // "data" for the data
-        QString m_basePath;
+        Soprano::Model* m_model;
+        const Soprano::Backend* m_backend;
 
+        // only used during opening
+        // ------------------------------------------
+        ModelCopyJob* m_modelCopyJob;
         const Soprano::Backend* m_oldStorageBackend;
         QString m_oldStoragePath;
 
-        Soprano::Model* m_model;
-        Nepomuk::CLuceneAnalyzer* m_analyzer;
-        Soprano::Index::CLuceneIndex* m_index;
-        Soprano::Index::IndexFilterModel* m_indexModel;
+        // the base path for the data. Will contain subfolder:
+        // "data" for the data
+        QString m_basePath;
+        // ------------------------------------------
     };
 
     typedef QMap<QString, Repository*> RepositoryMap;
