@@ -27,8 +27,20 @@ class QDir;
 
 #include <KUrl>
 
+/** 
+ contains global options 
+*/
+class GlobalOptions {
+public: 
+    GlobalOptions() : useCategories(true) {}
+
+    bool useCategories;
+};
+
+extern GlobalOptions globalOptions;
+
 bool removeDirectory(const QString& aDir);
-QString getStartMenuPath(bool bAllUsers=true);
+QString getStartMenuPath(bool bAllUsers=false);
 QString getKDEStartMenuPath();
 bool generateMenuEntries(QList<LinkFile> &files, const KUrl &url, const QString &relPathTranslated=QString());
 void updateStartMenuLinks();
